'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
var _bind = Function.prototype.bind;
exports.marksy = marksy;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _marked = require('marked');

var _marked2 = _interopRequireDefault(_marked);

var _he = require('he');

var _he2 = _interopRequireDefault(_he);

var _babelStandalone = require('babel-standalone');

var _CodeComponent = require('./CodeComponent');

var _CodeComponent2 = _interopRequireDefault(_CodeComponent);

var renderer = new _marked2['default'].Renderer();

function marksy() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  options.components = options.components || {};

  var inlineIds = 0;
  var keys = 0;
  var inlines = {};
  var result = [];
  var toc = [];

  // Converts inline IDs to actual elements
  function createBlockContent(content) {
    var textWithInlines = content.split(/(\{\{.*?\}\})/);
    content = textWithInlines.map(function (text) {
      var inline = text.match(/\{\{(.*)\}\}/);
      if (inline) {
        return inlines[inline[1]];
      } else {
        if (text != '') {
          return _he2['default'].decode(text);
        }
      }
    });

    return content;
  }

  function getTocPosition(toc, level) {
    var currentLevel = toc.children;
    while (true) {
      if (!currentLevel.length || currentLevel[currentLevel.length - 1].level === level) {
        return currentLevel;
      } else {
        currentLevel = currentLevel[currentLevel.length - 1].children;
      }
    }
  }

  renderer.code = function (code, language) {
    var props = {
      key: keys++,
      language: language,
      code: code
    };

    if (options.code) {
      result.push(_react2['default'].createElement(options.code, props));
    } else {
      result.push(_react2['default'].createElement(_CodeComponent2['default'], props));
    }
  };

  renderer.blockquote = function (text) {
    var count = text.split(/(\{\{.*?\}\})/).filter(function (n) {
      return n != "";
    });

    count.forEach(function () {
      result.pop();
    });

    result.push(_react2['default'].createElement(options.blockquote || 'blockquote', { key: keys++ }, createBlockContent(text)));
  };

  renderer.html = function (html) {
    try {
      (function () {
        var code = (0, _babelStandalone.transform)(html, {
          presets: ['react']
        }).code;
        var components = Object.keys(options.components).map(function (key) {
          return options.components[key];
        });

        result.push(_react2['default'].createElement(function () {
          return new (_bind.apply(Function, [null].concat(['React'], _toConsumableArray(Object.keys(options.components)), ['return ' + code])))().apply(undefined, [_react2['default']].concat(_toConsumableArray(components)));
        }));
      })();
    } catch (e) {}
  };

  renderer.heading = function (text, level) {
    var type = 'h' + level;
    type = options[type] || type;
    var id = text.replace(/\s/g, '-').toLowerCase();
    var lastToc = toc[toc.length - 1];
    if (!lastToc || lastToc.level > level) {
      toc.push({
        id: id,
        title: text,
        level: level,
        children: []
      });
    } else {
      var tocPosition = getTocPosition(lastToc, level);
      tocPosition.push({
        id: id,
        title: text,
        level: level,
        children: []
      });
    }
    var inId = inlineIds++;
    inlines[inId] = _react2['default'].createElement(type, {
      key: keys++,
      id: id
    }, createBlockContent(text));
    result.push(inlines[inId]);
    return '{{' + inId + '}}';
  };

  renderer.hr = function () {
    result.push(_react2['default'].createElement(options.hr || 'hr', { key: keys++ }));
  };

  renderer.list = function (body, ordered) {
    var id = inlineIds++;
    inlines[id] = _react2['default'].createElement(ordered ? options.ol || 'ol' : options.ul || 'ul', { key: keys++ }, createBlockContent(body));
    result.push(inlines[id]);
    return '{{' + id + '}}';
  };

  renderer.listitem = function (text) {
    var id = inlineIds++;
    inlines[id] = _react2['default'].createElement(options.li || 'li', { key: keys++ }, createBlockContent(text));
    return '{{' + id + '}}';
  };

  renderer.paragraph = function (text) {
    var id = inlineIds++;
    inlines[id] = _react2['default'].createElement(options.p || 'p', { key: keys++ }, createBlockContent(text));
    result.push(inlines[id]);
    return '{{' + id + '}}';
  };

  renderer.table = function (header, body) {
    var id = inlineIds++;
    inlines[id] = _react2['default'].createElement(options.table || 'table', { key: keys++ }, _react2['default'].createElement(options.thead || 'thead', null, createBlockContent(header)), _react2['default'].createElement(options.tbody || 'tbody', null, createBlockContent(body)));
    result.push(inlines[id]);
    return '{{' + id + '}}';
  };

  renderer.thead = function (content) {
    var id = inlineIds++;
    inlines[id] = _react2['default'].createElement(options.thead || 'thead', { key: keys++ }, createBlockContent(content));
    return '{{' + id + '}}';
  };

  renderer.tbody = function (content) {
    var id = inlineIds++;
    inlines[id] = _react2['default'].createElement(options.tbody || 'tbody', { key: keys++ }, createBlockContent(content));
    return '{{' + id + '}}';
  };

  renderer.tablerow = function (content) {
    var id = inlineIds++;
    inlines[id] = _react2['default'].createElement(options.tr || 'tr', { key: keys++ }, createBlockContent(content));
    return '{{' + id + '}}';
  };

  renderer.tablecell = function (content, flags) {
    var id = inlineIds++;
    var props = flags.align ? { className: 'text-' + flags.align } : { key: keys++ };
    inlines[id] = _react2['default'].createElement(flags.header ? options.th || 'th' : options.td || 'td', props, createBlockContent(content));
    return '{{' + id + '}}';
  };

  renderer.link = function (href, title, text) {
    var id = inlineIds++;
    inlines[id] = _react2['default'].createElement(options.a || 'a', {
      href: href,
      title: title,
      key: keys++,
      target: 'new'
    }, createBlockContent(text));
    return '{{' + id + '}}';
  };

  renderer.strong = function (text) {
    var id = inlineIds++;
    inlines[id] = _react2['default'].createElement(options.strong || 'strong', { key: keys++ }, createBlockContent(text));
    return '{{' + id + '}}';
  };

  renderer.em = function (text) {
    var id = inlineIds++;
    inlines[id] = _react2['default'].createElement(options.em || 'em', { key: keys++ }, createBlockContent(text));
    return '{{' + id + '}}';
  };

  renderer.codespan = function (text) {
    var id = inlineIds++;
    inlines[id] = _react2['default'].createElement(options.codespan || 'code', { key: keys++ }, _he2['default'].decode(text));
    return '{{' + id + '}}';
  };

  renderer.br = function () {
    var id = inlineIds++;
    inlines[id] = _react2['default'].createElement(options.br || 'br', { key: keys++ });
    return '{{' + id + '}}';
  };

  renderer.del = function (text) {
    var id = inlineIds++;
    inlines[id] = _react2['default'].createElement(options.del || 'del', { key: keys++ }, _he2['default'].decode(text));
    return '{{' + id + '}}';
  };

  renderer.image = function (href, title, text) {
    var id = inlineIds++;
    inlines[id] = _react2['default'].createElement(options.img || 'img', { src: href, alt: title, key: keys++ });
    return '{{' + id + '}}';
  };

  return function compile(content) {
    result = [];
    toc = [];
    inlines = {};
    keys = 0;

    (0, _marked2['default'])(content, { renderer: renderer, smartypants: true });

    return {
      tree: result,
      toc: toc
    };
  };
}

exports['default'] = function (options) {
  return marksy(options);
};

;
//# sourceMappingURL=index.js.map